#!/usr/bin/env python

"""
This is the setup for the S1S2 Actors study.

"""
import os
EXAMPLE_DESCRIPTIVE_NAME = 'S1-S2 Actors Simulation'
EXAMPLE_AUTHOR = 'Joyce Reimer <joyce.reimer@usask.ca>'
EXAMPLE_DIR = os.path.dirname(__file__)
GUIinclude = True

import sys
from datetime import date

from carputils import settings
from carputils import tools
from carputils import mesh
import numpy as np
from numpy import array as nplist

def parser():
    parser = tools.standard_parser()
    group  = parser.add_argument_group('experiment specific options')
    group.add_argument('--duration',
                        type = float,
                        default = 300.,
                        help = 'Duration of simulation in [ms]')
    group.add_argument('--S1-strength',
                       type = float,
                       default = 0.,
                       help = 'pick S1 extracellular current stimulus strength in [uA/cm^3]')
    group.add_argument('--S1-dur',
                        type = float,
                        default = 2.,
                        help = 'pick S1 current stimulus duration in [ms] (default: 2.)')
    group.add_argument('--S1-start',
                        type = float,
                        default = 0.0,
                        help = 'pick S1 start time')
    group.add_argument('--S2-strength',
                       type = float,
                       default = 0.,
                       help = 'pick S2 extracellular current stimulus strength in [uA/cm^3]')
    group.add_argument('--S2-dur',
                        type = float,
                        default = 2.,
                        help = 'pick S2 current stimulus duration in [ms] (default: 2.)')
    group.add_argument('--S2-start',
                        type = float,
                        default = 235.0,
                        help = 'pick S2 start time')

    return parser

def jobID(args):
    """
    Generate name of top level output directory.
    """
    today = date.today()
    return '{}_XCstim_{}_np{}'.format(today.isoformat(),
                                       args.flv, args.np)

@tools.carpexample(parser, jobID)
def run(args, job):

    # Generate mesh
    # Block which is thin in z direction
    geom = mesh.Block(size=(4.0, 0.625, 0.025),res=(0.025)) #in mm
    #geom = mesh.Block(size=(1.0, 1.0, 2.0)) #in mm
    # Set fibre angle to 0, sheet angle to 0
    geom.set_fibres(0, 0, 90, 90)
    geom.corner_at_origin()
    # Generate and return base name
    meshname = mesh.generate(geom)

    # define electrode locations
    #Test electrode is in bottom left corner. Has the same area as our experimental data
    # and has a z-thickness the size of dx. 
    
    E1_lower_bound = nplist([ 0.0, 0.0, 2.0]) #in mm 
    E2_lower_bound = nplist([ 0.0, 0.0, 2.0]) 
    # Last number should always be z thickness; first two numbers should be 0 if corner at origin.
    
    E1_upper_bound = nplist([ 0.175, 0.175, 2.0]) #in mm 
    E2_upper_bound = nplist([ 0.175, 0.175, 2.0]) #in mm 
    #With these dimensions and default resolution of 0.1mm, we get electrode area = .225 x .225 = 0.05 mm^2. 
    #Last number should always be z thickness.
   
    #S1 and S2 electrodes:
    lstm = mesh.block_region(geom, 'stimulus', 0, E1_lower_bound, E1_upper_bound, False)
    lstm2 = mesh.block_region(geom, 'stimulus', 1, E2_lower_bound, E2_upper_bound, False)


    #Reference/ground electrode is behind domain.
    rstm = mesh.block_boundary_condition(geom, 'stimulus', 2, 'z', lower=True, verbose=not args.silent) 
    
    
    del geom

    #astm = []
    #mod = []

    # tags generated with the above command
    IntraTags = [1]
    ExtraTags = [1]

    lstm += ['-stimulus[0].stimtype', '1',
            '-stimulus[0].strength', args.S1_strength, #3e7 uA/cm^3 worked
            '-stimulus[0].duration', args.S1_dur,
            '-stimulus[0].start',    args.S1_start]
    lstm2 +=['-stimulus[1].stimtype', '1',
            '-stimulus[1].strength', args.S2_strength,
            '-stimulus[1].duration', args.S2_dur,
            '-stimulus[1].start',    args.S2_start]


    # Add all the non-general arguments
    cmd = tools.carp_cmd("/code/simulations/input/s1.par")

    cmd += ['-meshname', meshname,
            '-simID',    job.ID,
            '-tend',     args.duration]
    cmd += tools.gen_physics_opts(ExtraTags=ExtraTags, IntraTags=IntraTags)

    # add PDE solver options
    cmd += lstm + lstm2 + rstm

    if args.visualize:
        cmd += ['-gridout_i', 3,
                '-gridout_e', 3]
    
    # Run simulations
    job.carp(cmd)

    
if __name__ == '__main__':
    run()